
-- linkage
IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'linkage' AND TABLE_SCHEMA = 'dbo')
    DROP TABLE dbo.linkage
GO


-- partner
IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'partner' AND TABLE_SCHEMA = 'dbo')
    DROP TABLE dbo.partner
GO
